# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .v_1_vision_step_results_item_presence_absence_blue_range import V1VisionStepResultsItemPresenceAbsenceBlueRange
from .v_1_vision_step_results_item_presence_absence_grayscale_range import (
    V1VisionStepResultsItemPresenceAbsenceGrayscaleRange,
)
from .v_1_vision_step_results_item_presence_absence_green_range import V1VisionStepResultsItemPresenceAbsenceGreenRange
from .v_1_vision_step_results_item_presence_absence_hue_range import V1VisionStepResultsItemPresenceAbsenceHueRange
from .v_1_vision_step_results_item_presence_absence_red_range import V1VisionStepResultsItemPresenceAbsenceRedRange
from .v_1_vision_step_results_item_presence_absence_saturation_range import (
    V1VisionStepResultsItemPresenceAbsenceSaturationRange,
)
from .v_1_vision_step_results_item_presence_absence_value_range import V1VisionStepResultsItemPresenceAbsenceValueRange


class V1VisionStepResultsItemPresenceAbsence(UniversalBaseModel):
    blue_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceBlueRange],
        FieldMetadata(alias="blueRange"),
        pydantic.Field(alias="blueRange"),
    ] = None
    check_for_absence: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="checkForAbsence"), pydantic.Field(alias="checkForAbsence")
    ] = None
    grayscale_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceGrayscaleRange],
        FieldMetadata(alias="grayscaleRange"),
        pydantic.Field(alias="grayscaleRange"),
    ] = None
    green_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceGreenRange],
        FieldMetadata(alias="greenRange"),
        pydantic.Field(alias="greenRange"),
    ] = None
    hue_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceHueRange],
        FieldMetadata(alias="hueRange"),
        pydantic.Field(alias="hueRange"),
    ] = None
    red_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceRedRange],
        FieldMetadata(alias="redRange"),
        pydantic.Field(alias="redRange"),
    ] = None
    saturation_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceSaturationRange],
        FieldMetadata(alias="saturationRange"),
        pydantic.Field(alias="saturationRange"),
    ] = None
    score: typing.Optional[int] = None
    threshold: typing.Optional[int] = None
    value_range: typing_extensions.Annotated[
        typing.Optional[V1VisionStepResultsItemPresenceAbsenceValueRange],
        FieldMetadata(alias="valueRange"),
        pydantic.Field(alias="valueRange"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .vehicle_stats_aux_input_time import VehicleStatsAuxInputTime
from .vehicle_stats_decorations import VehicleStatsDecorations
from .vehicle_stats_engine_immobilizer_with_decoration_state import VehicleStatsEngineImmobilizerWithDecorationState


class VehicleStatsEngineImmobilizerWithDecoration(UniversalBaseModel):
    """
    Data for the engine immobilizer
    """

    connected: bool = pydantic.Field()
    """
    Whether the engine immobilizer is connected or not
    """

    decorations: typing.Optional[VehicleStatsDecorations] = None
    state: VehicleStatsEngineImmobilizerWithDecorationState = pydantic.Field()
    """
    The state of the engine immobilizer. Valid values: `ignition_disabled`, `ignition_enabled`. This stat type will only return states of our first Engine Immobilizer Hardware (ACC-EI). Please use <a href="https://developers.samsara.com/reference/getengineimmobilizerstates" target="_blank">Get engine immobilizer states</a> to get states for both Engine Immobilizer Hardware versions (incl. HW-EI21).
    """

    time: VehicleStatsAuxInputTime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

DriverRulesetObjectResponseBodyCycle = typing.Union[
    typing.Literal[
        "USA 60 hour / 7 day",
        "USA 70 hour / 8 day",
        "AK 80 hour / 8 day",
        "AK 70 hour / 7 day",
        "CA 80 hour / 8 day",
        "CA 112 hour / 8 day",
        "FL 80 hour / 8 day",
        "FL 70 hour / 7 day",
        "NE 80 hour / 8 day",
        "NE 70 hour / 7 day",
        "NC 80 hour / 8 day",
        "NC 70 hour / 7 day",
        "OK 70 hour / 8 day",
        "OK 60 hour / 7 day",
        "OR 80 hour / 8 day",
        "OR 70 hour / 7 day",
        "SC 80 hour / 8 day",
        "SC 70 hour / 7 day",
        "TX 70 hour / 7 day",
        "WI 80 hour / 8 day",
        "WI 70 hour / 7 day",
        "Canada South Cycle 1 (70 hour / 7 day)",
        "Canada South Cycle 2 (120 hour / 14 day)",
        "Canada North Cycle 1 (80 hour / 7 day)",
        "Canada North Cycle 2 (120 hour / 14 day)",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

TrainingCourseResponseObjectResponseBodyStatus = typing.Union[
    typing.Literal["published", "deleted", "archived", "unknown"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .hub_custom_property_object_response_body import HubCustomPropertyObjectResponseBody
from .list_hub_custom_properties_response_pagination_response_body import (
    ListHubCustomPropertiesResponsePaginationResponseBody,
)


class HubCustomPropertiesListHubCustomPropertiesResponseBody(UniversalBaseModel):
    data: typing.List[HubCustomPropertyObjectResponseBody] = pydantic.Field()
    """
    Hub custom properties data
    """

    pagination: ListHubCustomPropertiesResponsePaginationResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..types.inline_response_2005 import InlineResponse2005
from ..types.inline_response_2006 import InlineResponse2006

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMessagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def v_1_get_messages(
        self,
        *,
        end_ms: typing.Optional[int] = None,
        duration_ms: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[InlineResponse2005]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get all messages.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        end_ms : typing.Optional[int]
            Time in unix milliseconds that represents the end of time range of messages to return. Used in combination with durationMs. Defaults to now.

        duration_ms : typing.Optional[int]
            Time in milliseconds that represents the duration before endMs to query. Defaults to 24 hours.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InlineResponse2005]
            Returns the fetched messages from most recently sent to least recently sent.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/fleet/messages",
            method="GET",
            params={
                "endMs": end_ms,
                "durationMs": duration_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2005,
                    parse_obj_as(
                        type_=InlineResponse2005,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def v_1_create_messages(
        self, *, driver_ids: typing.Sequence[int], text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[InlineResponse2006]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Send a message to a list of driver ids.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_ids : typing.Sequence[int]
            IDs of the drivers for whom the messages are sent to.

        text : str
            The text sent in the message. Max 2500 characters allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[InlineResponse2006]
            Returns the created messages.
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/fleet/messages",
            method="POST",
            json={
                "driverIds": driver_ids,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2006,
                    parse_obj_as(
                        type_=InlineResponse2006,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMessagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def v_1_get_messages(
        self,
        *,
        end_ms: typing.Optional[int] = None,
        duration_ms: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[InlineResponse2005]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Get all messages.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        end_ms : typing.Optional[int]
            Time in unix milliseconds that represents the end of time range of messages to return. Used in combination with durationMs. Defaults to now.

        duration_ms : typing.Optional[int]
            Time in milliseconds that represents the duration before endMs to query. Defaults to 24 hours.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InlineResponse2005]
            Returns the fetched messages from most recently sent to least recently sent.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/fleet/messages",
            method="GET",
            params={
                "endMs": end_ms,
                "durationMs": duration_ms,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2005,
                    parse_obj_as(
                        type_=InlineResponse2005,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def v_1_create_messages(
        self, *, driver_ids: typing.Sequence[int], text: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[InlineResponse2006]:
        """
        <n class="warning">
        <nh>
        <i class="fa fa-exclamation-circle"></i>
        This endpoint is still on our legacy API.
        </nh>
        </n>

        Send a message to a list of driver ids.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Write Messages** under the Driver Workflow category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        driver_ids : typing.Sequence[int]
            IDs of the drivers for whom the messages are sent to.

        text : str
            The text sent in the message. Max 2500 characters allowed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[InlineResponse2006]
            Returns the created messages.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/fleet/messages",
            method="POST",
            json={
                "driverIds": driver_ids,
                "text": text,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    InlineResponse2006,
                    parse_obj_as(
                        type_=InlineResponse2006,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .depreciation_transaction_money_object_response_body import DepreciationTransactionMoneyObjectResponseBody
from .depreciation_transaction_object_response_body_transaction_type import (
    DepreciationTransactionObjectResponseBodyTransactionType,
)


class DepreciationTransactionObjectResponseBody(UniversalBaseModel):
    """
    A depreciation or adjustment transaction
    """

    amount: DepreciationTransactionMoneyObjectResponseBody
    asset_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="assetId"),
        pydantic.Field(
            alias="assetId",
            description="Samsara asset ID. Can be used with /fleet/assets/{id} to retrieve asset details.",
        ),
    ]
    cost_center_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="costCenterId"),
        pydantic.Field(
            alias="costCenterId",
            description='The UUID of the cost center assigned to the asset. To retrieve cost center names: (1) Call GET /attributes?entityType=asset to find the "Asset Cost Center" attribute and its UUID, (2) Call GET /attributes/{that-uuid}?entityType=asset to get the list of cost center values with their UUIDs and names. Match the costCenterId from this response to the uuid field in the attributeValues array to get the cost center name.',
        ),
    ] = None
    created_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAt"),
        pydantic.Field(
            alias="createdAt", description="When the transaction record was created in the system. RFC 3339 format."
        ),
    ]
    event_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="eventTime"),
        pydantic.Field(
            alias="eventTime",
            description="When the depreciation or adjustment occurred from a financial/accounting perspective (e.g., end of depreciation period, or date of revaluation). This may differ from createdAt if the transaction is recorded after the fact. RFC 3339 format.",
        ),
    ]
    id: str = pydantic.Field()
    """
    The unique UUID of the transaction.
    """

    note: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional description.
    """

    transaction_type: typing_extensions.Annotated[
        DepreciationTransactionObjectResponseBodyTransactionType,
        FieldMetadata(alias="transactionType"),
        pydantic.Field(
            alias="transactionType",
            description="Transaction type: depreciation, adjustment, or unknown. Unknown transaction types should be handled gracefully for forward compatibility.  Valid values: `depreciation`, `adjustment`, `unknown`",
        ),
    ]
    updated_at: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAt"),
        pydantic.Field(
            alias="updatedAt",
            description="When the transaction record was last modified in the system (use for incremental sync). RFC 3339 format.",
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..types.equipment_list_response import EquipmentListResponse
from ..types.equipment_locations_list_response import EquipmentLocationsListResponse
from ..types.equipment_locations_response import EquipmentLocationsResponse
from ..types.equipment_response import EquipmentResponse
from ..types.equipment_stats_list_response import EquipmentStatsListResponse
from ..types.equipment_stats_response import EquipmentStatsResponse
from .types.get_equipment_stats_feed_request_types_item import GetEquipmentStatsFeedRequestTypesItem
from .types.get_equipment_stats_history_request_types_item import GetEquipmentStatsHistoryRequestTypesItem
from .types.get_equipment_stats_request_types_item import GetEquipmentStatsRequestTypesItem


class RawEquipmentClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_equipment(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentListResponse]:
        """
        Returns a list of all equipment in an organization.

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentListResponse]
            List of all equipment objects, and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentListResponse,
                    parse_obj_as(
                        type_=EquipmentListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment_locations(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentLocationsResponse]:
        """
        Returns last known locations for all equipment. This can be optionally filtered by tags or specific equipment IDs.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentLocationsResponse]
            The most recent equipment locations and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment/locations",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentLocationsResponse,
                    parse_obj_as(
                        type_=EquipmentLocationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment_locations_feed(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentLocationsListResponse]:
        """
        Follow a continuous feed of all equipment locations.

        Your first call to this endpoint will provide you with the most recent location for each unit of equipment and a `pagination` object that contains an `endCursor`.

        You can provide the `endCursor` to subsequent calls via the `after` parameter. The response will contain any equipment location updates since that `endCursor`.

        If `hasNextPage` is `false`, no updates are readily available yet. We'd suggest waiting a minimum of 5 seconds before requesting updates.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentLocationsListResponse]
            The feed of equipment locations and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment/locations/feed",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentLocationsListResponse,
                    parse_obj_as(
                        type_=EquipmentLocationsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment_locations_history(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentLocationsListResponse]:
        """
        Returns historical equipment locations during the given time range. This can be optionally filtered by tags or specific equipment IDs.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentLocationsListResponse]
            Historical equipment locations and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment/locations/history",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentLocationsListResponse,
                    parse_obj_as(
                        type_=EquipmentLocationsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment_stats(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        types: typing.Optional[
            typing.Union[GetEquipmentStatsRequestTypesItem, typing.Sequence[GetEquipmentStatsRequestTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentStatsResponse]:
        """
        Returns the last known stats for all equipment. This can be optionally filtered by tags or specific equipment IDs.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        types : typing.Optional[typing.Union[GetEquipmentStatsRequestTypesItem, typing.Sequence[GetEquipmentStatsRequestTypesItem]]]
            The types of equipment stats you want to query. Currently, you may submit up to 4 types.

            - `engineRpm`: The revolutions per minute of the engine.
            - `fuelPercents`: The percent of fuel in the unit of equipment.
            - `obdEngineSeconds`: The number of seconds the engine has been running as reported directly from on-board diagnostics. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
            - `gatewayEngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the asset gateway has been receiving power with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations:
              - AG24/AG26/AG46P + APWR cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required)
              - AG52 + BPWR/BEQP cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required).
            - `gatewayJ1939EngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power via J1939/CAT cable and an offset provided manually through the Samsara cloud dashboard.
            - `obdEngineStates`: The state of the engine read from on-board diagnostics. Can be `Off`, `On`, or `Idle`.
            - `gatewayEngineStates`: An approximation of engine state based on readings the AG26 receives from the aux/digio cable. Can be `Off` or `On`.
            - `gpsOdometerMeters`: An approximation of odometer reading based on GPS calculations since the AG26 was activated, and a manual odometer offset provided in the Samsara cloud dashboard. Valid values: `Off`, `On`.
            - `gps`: GPS data including lat/long, heading, speed, address book entry (if exists), and a reverse geocoded address.
            - `engineTotalIdleTimeMinutes`: Total time in minutes that the engine has been idling.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentStatsResponse]
            The most recent equipment stats and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment/stats",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
                "types": types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentStatsResponse,
                    parse_obj_as(
                        type_=EquipmentStatsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment_stats_feed(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        types: typing.Optional[
            typing.Union[GetEquipmentStatsFeedRequestTypesItem, typing.Sequence[GetEquipmentStatsFeedRequestTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentStatsListResponse]:
        """
        Follow a continuous feed of all equipment stats.

        Your first call to this endpoint will provide you with the most recent stats for each unit of equipment and a `pagination` object that contains an `endCursor`.

        You can provide the `endCursor` to subsequent calls via the `after` parameter. The response will contain any equipment stats updates since that `endCursor`.

        If `hasNextPage` is `false`, no updates are readily available yet. Each stat type has a different refresh rate, but in general we'd suggest waiting a minimum of 5 seconds before requesting updates.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        types : typing.Optional[typing.Union[GetEquipmentStatsFeedRequestTypesItem, typing.Sequence[GetEquipmentStatsFeedRequestTypesItem]]]
            The types of equipment stats you want to query. Currently, you may submit up to 4 types.

            - `engineRpm`: The revolutions per minute of the engine.
            - `fuelPercents`: The percent of fuel in the unit of equipment.
            - `obdEngineSeconds`: The number of seconds the engine has been running as reported directly from on-board diagnostics. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
            - `gatewayEngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the asset gateway has been receiving power with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations:
              - AG24/AG26/AG46P + APWR cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required)
              - AG52 + BPWR/BEQP cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required).
            - `gatewayJ1939EngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power via J1939/CAT cable and an offset provided manually through the Samsara cloud dashboard.
            - `obdEngineStates`: The state of the engine read from on-board diagnostics. Can be `Off`, `On`, or `Idle`.
            - `gatewayEngineStates`: An approximation of engine state based on readings the AG26 receives from the aux/digio cable. Can be `Off` or `On`.
            - `gpsOdometerMeters`: An approximation of odometer reading based on GPS calculations since the AG26 was activated, and a manual odometer offset provided in the Samsara cloud dashboard. Valid values: `Off`, `On`.
            - `gps`: GPS data including lat/long, heading, speed, address book entry (if exists), and a reverse geocoded address.
            - `engineTotalIdleTimeMinutes`: Total time in minutes that the engine has been idling.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentStatsListResponse]
            The feed of equipment stats and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment/stats/feed",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
                "types": types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentStatsListResponse,
                    parse_obj_as(
                        type_=EquipmentStatsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment_stats_history(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        types: typing.Optional[
            typing.Union[
                GetEquipmentStatsHistoryRequestTypesItem, typing.Sequence[GetEquipmentStatsHistoryRequestTypesItem]
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EquipmentStatsListResponse]:
        """
        Returns historical equipment status during the given time range. This can be optionally filtered by tags or specific equipment IDs.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        types : typing.Optional[typing.Union[GetEquipmentStatsHistoryRequestTypesItem, typing.Sequence[GetEquipmentStatsHistoryRequestTypesItem]]]
            The types of equipment stats you want to query. Currently, you may submit up to 4 types.

            - `engineRpm`: The revolutions per minute of the engine.
            - `fuelPercents`: The percent of fuel in the unit of equipment.
            - `obdEngineSeconds`: The number of seconds the engine has been running as reported directly from on-board diagnostics. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
            - `gatewayEngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the asset gateway has been receiving power with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations:
              - AG24/AG26/AG46P + APWR cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required)
              - AG52 + BPWR/BEQP cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required).
            - `gatewayJ1939EngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power via J1939/CAT cable and an offset provided manually through the Samsara cloud dashboard.
            - `obdEngineStates`: The state of the engine read from on-board diagnostics. Can be `Off`, `On`, or `Idle`.
            - `gatewayEngineStates`: An approximation of engine state based on readings the AG26 receives from the aux/digio cable. Can be `Off` or `On`.
            - `gpsOdometerMeters`: An approximation of odometer reading based on GPS calculations since the AG26 was activated, and a manual odometer offset provided in the Samsara cloud dashboard. Valid values: `Off`, `On`.
            - `gps`: GPS data including lat/long, heading, speed, address book entry (if exists), and a reverse geocoded address.
            - `engineTotalIdleTimeMinutes`: Total time in minutes that the engine has been idling.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentStatsListResponse]
            Historical equipment stats and pagination information
        """
        _response = self._client_wrapper.httpx_client.request(
            "fleet/equipment/stats/history",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
                "types": types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentStatsListResponse,
                    parse_obj_as(
                        type_=EquipmentStatsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_equipment(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EquipmentResponse]:
        """
        Retrieves the unit of equipment with the given Samsara ID.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara ID of the Equipment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EquipmentResponse]
            The specified equipment object
        """
        _response = self._client_wrapper.httpx_client.request(
            f"fleet/equipment/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentResponse,
                    parse_obj_as(
                        type_=EquipmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawEquipmentClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_equipment(
        self,
        *,
        limit: typing.Optional[int] = None,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentListResponse]:
        """
        Returns a list of all equipment in an organization.

        Parameters
        ----------
        limit : typing.Optional[int]
            The limit for how many objects will be in the response. Default and max for this value is 512 objects.

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentListResponse]
            List of all equipment objects, and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment",
            method="GET",
            params={
                "limit": limit,
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentListResponse,
                    parse_obj_as(
                        type_=EquipmentListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment_locations(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentLocationsResponse]:
        """
        Returns last known locations for all equipment. This can be optionally filtered by tags or specific equipment IDs.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentLocationsResponse]
            The most recent equipment locations and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment/locations",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentLocationsResponse,
                    parse_obj_as(
                        type_=EquipmentLocationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment_locations_feed(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentLocationsListResponse]:
        """
        Follow a continuous feed of all equipment locations.

        Your first call to this endpoint will provide you with the most recent location for each unit of equipment and a `pagination` object that contains an `endCursor`.

        You can provide the `endCursor` to subsequent calls via the `after` parameter. The response will contain any equipment location updates since that `endCursor`.

        If `hasNextPage` is `false`, no updates are readily available yet. We'd suggest waiting a minimum of 5 seconds before requesting updates.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentLocationsListResponse]
            The feed of equipment locations and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment/locations/feed",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentLocationsListResponse,
                    parse_obj_as(
                        type_=EquipmentLocationsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment_locations_history(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentLocationsListResponse]:
        """
        Returns historical equipment locations during the given time range. This can be optionally filtered by tags or specific equipment IDs.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentLocationsListResponse]
            Historical equipment locations and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment/locations/history",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentLocationsListResponse,
                    parse_obj_as(
                        type_=EquipmentLocationsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment_stats(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        types: typing.Optional[
            typing.Union[GetEquipmentStatsRequestTypesItem, typing.Sequence[GetEquipmentStatsRequestTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentStatsResponse]:
        """
        Returns the last known stats for all equipment. This can be optionally filtered by tags or specific equipment IDs.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        types : typing.Optional[typing.Union[GetEquipmentStatsRequestTypesItem, typing.Sequence[GetEquipmentStatsRequestTypesItem]]]
            The types of equipment stats you want to query. Currently, you may submit up to 4 types.

            - `engineRpm`: The revolutions per minute of the engine.
            - `fuelPercents`: The percent of fuel in the unit of equipment.
            - `obdEngineSeconds`: The number of seconds the engine has been running as reported directly from on-board diagnostics. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
            - `gatewayEngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the asset gateway has been receiving power with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations:
              - AG24/AG26/AG46P + APWR cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required)
              - AG52 + BPWR/BEQP cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required).
            - `gatewayJ1939EngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power via J1939/CAT cable and an offset provided manually through the Samsara cloud dashboard.
            - `obdEngineStates`: The state of the engine read from on-board diagnostics. Can be `Off`, `On`, or `Idle`.
            - `gatewayEngineStates`: An approximation of engine state based on readings the AG26 receives from the aux/digio cable. Can be `Off` or `On`.
            - `gpsOdometerMeters`: An approximation of odometer reading based on GPS calculations since the AG26 was activated, and a manual odometer offset provided in the Samsara cloud dashboard. Valid values: `Off`, `On`.
            - `gps`: GPS data including lat/long, heading, speed, address book entry (if exists), and a reverse geocoded address.
            - `engineTotalIdleTimeMinutes`: Total time in minutes that the engine has been idling.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentStatsResponse]
            The most recent equipment stats and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment/stats",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
                "types": types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentStatsResponse,
                    parse_obj_as(
                        type_=EquipmentStatsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment_stats_feed(
        self,
        *,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        types: typing.Optional[
            typing.Union[GetEquipmentStatsFeedRequestTypesItem, typing.Sequence[GetEquipmentStatsFeedRequestTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentStatsListResponse]:
        """
        Follow a continuous feed of all equipment stats.

        Your first call to this endpoint will provide you with the most recent stats for each unit of equipment and a `pagination` object that contains an `endCursor`.

        You can provide the `endCursor` to subsequent calls via the `after` parameter. The response will contain any equipment stats updates since that `endCursor`.

        If `hasNextPage` is `false`, no updates are readily available yet. Each stat type has a different refresh rate, but in general we'd suggest waiting a minimum of 5 seconds before requesting updates.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        types : typing.Optional[typing.Union[GetEquipmentStatsFeedRequestTypesItem, typing.Sequence[GetEquipmentStatsFeedRequestTypesItem]]]
            The types of equipment stats you want to query. Currently, you may submit up to 4 types.

            - `engineRpm`: The revolutions per minute of the engine.
            - `fuelPercents`: The percent of fuel in the unit of equipment.
            - `obdEngineSeconds`: The number of seconds the engine has been running as reported directly from on-board diagnostics. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
            - `gatewayEngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the asset gateway has been receiving power with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations:
              - AG24/AG26/AG46P + APWR cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required)
              - AG52 + BPWR/BEQP cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required).
            - `gatewayJ1939EngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power via J1939/CAT cable and an offset provided manually through the Samsara cloud dashboard.
            - `obdEngineStates`: The state of the engine read from on-board diagnostics. Can be `Off`, `On`, or `Idle`.
            - `gatewayEngineStates`: An approximation of engine state based on readings the AG26 receives from the aux/digio cable. Can be `Off` or `On`.
            - `gpsOdometerMeters`: An approximation of odometer reading based on GPS calculations since the AG26 was activated, and a manual odometer offset provided in the Samsara cloud dashboard. Valid values: `Off`, `On`.
            - `gps`: GPS data including lat/long, heading, speed, address book entry (if exists), and a reverse geocoded address.
            - `engineTotalIdleTimeMinutes`: Total time in minutes that the engine has been idling.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentStatsListResponse]
            The feed of equipment stats and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment/stats/feed",
            method="GET",
            params={
                "after": after,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
                "types": types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentStatsListResponse,
                    parse_obj_as(
                        type_=EquipmentStatsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment_stats_history(
        self,
        *,
        start_time: str,
        end_time: str,
        after: typing.Optional[str] = None,
        parent_tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        tag_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        equipment_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        types: typing.Optional[
            typing.Union[
                GetEquipmentStatsHistoryRequestTypesItem, typing.Sequence[GetEquipmentStatsHistoryRequestTypesItem]
            ]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EquipmentStatsListResponse]:
        """
        Returns historical equipment status during the given time range. This can be optionally filtered by tags or specific equipment IDs.

         <b>Rate limit:</b> 150 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment Statistics** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        start_time : str
            A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : str
            An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        after : typing.Optional[str]
            If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        parent_tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of parent tag IDs, for use by orgs with tag hierarchies. Specifying a parent tag will implicitly include all descendent tags of the parent tag. Example: `parentTagIds=345,678`

        tag_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of tag IDs. Example: `tagIds=1234,5678`

        equipment_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            A filter on the data based on this comma-separated list of equipment IDs. Example: `equipmentIds=1234,5678`

        types : typing.Optional[typing.Union[GetEquipmentStatsHistoryRequestTypesItem, typing.Sequence[GetEquipmentStatsHistoryRequestTypesItem]]]
            The types of equipment stats you want to query. Currently, you may submit up to 4 types.

            - `engineRpm`: The revolutions per minute of the engine.
            - `fuelPercents`: The percent of fuel in the unit of equipment.
            - `obdEngineSeconds`: The number of seconds the engine has been running as reported directly from on-board diagnostics. This is supported with the following hardware configurations: AG24/AG26 + AOPEN/A9PIN/ACT9/ACT14.
            - `gatewayEngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the asset gateway has been receiving power with an offset provided manually through the Samsara cloud dashboard. This is supported with the following hardware configurations:
              - AG24/AG26/AG46P + APWR cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required)
              - AG52 + BPWR/BEQP cable ([Auxiliary engine configuration](https://kb.samsara.com/hc/en-us/articles/360043040512-Auxiliary-Inputs#UUID-d514abff-d10a-efaf-35d9-e10fa6c4888d) required).
            - `gatewayJ1939EngineSeconds`: An approximation of the number of seconds the engine has been running since it was new, based on the amount of time the AG26 device is receiving power via J1939/CAT cable and an offset provided manually through the Samsara cloud dashboard.
            - `obdEngineStates`: The state of the engine read from on-board diagnostics. Can be `Off`, `On`, or `Idle`.
            - `gatewayEngineStates`: An approximation of engine state based on readings the AG26 receives from the aux/digio cable. Can be `Off` or `On`.
            - `gpsOdometerMeters`: An approximation of odometer reading based on GPS calculations since the AG26 was activated, and a manual odometer offset provided in the Samsara cloud dashboard. Valid values: `Off`, `On`.
            - `gps`: GPS data including lat/long, heading, speed, address book entry (if exists), and a reverse geocoded address.
            - `engineTotalIdleTimeMinutes`: Total time in minutes that the engine has been idling.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentStatsListResponse]
            Historical equipment stats and pagination information
        """
        _response = await self._client_wrapper.httpx_client.request(
            "fleet/equipment/stats/history",
            method="GET",
            params={
                "after": after,
                "startTime": start_time,
                "endTime": end_time,
                "parentTagIds": parent_tag_ids,
                "tagIds": tag_ids,
                "equipmentIds": equipment_ids,
                "types": types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentStatsListResponse,
                    parse_obj_as(
                        type_=EquipmentStatsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_equipment(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EquipmentResponse]:
        """
        Retrieves the unit of equipment with the given Samsara ID.

         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        To use this endpoint, select **Read Equipment** under the Equipment category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>

        Parameters
        ----------
        id : str
            Samsara ID of the Equipment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EquipmentResponse]
            The specified equipment object
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"fleet/equipment/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EquipmentResponse,
                    parse_obj_as(
                        type_=EquipmentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.driver_vehicle_assignments_v_2_create_driver_vehicle_assignment_response_body import (
    DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody,
)
from ..types.driver_vehicle_assignments_v_2_get_driver_vehicle_assignments_response_body import (
    DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody,
)
from ..types.driver_vehicle_assignments_v_2_update_driver_vehicle_assignment_response_body import (
    DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody,
)
from ..types.patch_driver_vehicle_assignments_v_2_request_body_metadata_request_body import (
    PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
)
from ..types.post_driver_vehicle_assignments_v_2_request_body_metadata_request_body import (
    PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody,
)
from .raw_client import AsyncRawDriverVehicleAssignmentsClient, RawDriverVehicleAssignmentsClient
from .types.get_driver_vehicle_assignments_request_assignment_type import (
    GetDriverVehicleAssignmentsRequestAssignmentType,
)
from .types.get_driver_vehicle_assignments_request_filter_by import GetDriverVehicleAssignmentsRequestFilterBy

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DriverVehicleAssignmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDriverVehicleAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDriverVehicleAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDriverVehicleAssignmentsClient
        """
        return self._raw_client

    def get_driver_vehicle_assignments(
        self,
        *,
        filter_by: GetDriverVehicleAssignmentsRequestFilterBy,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_tag_ids: typing.Optional[str] = None,
        vehicle_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        assignment_type: typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody:
        """
        Get all driver-vehicle assignments for the requested drivers or vehicles in the requested time range. To fetch driver-vehicle assignments out of the vehicle trips' time ranges, assignmentType needs to be specified. Note: this endpoint replaces past endpoints to fetch assignments by driver or by vehicle. Visit [this migration guide](https://developers.samsara.com/docs/migrating-from-driver-vehicle-assignment-or-vehicle-driver-assignment-endpoints) for more information.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        filter_by : GetDriverVehicleAssignmentsRequestFilterBy
            Option to filter by drivers or vehicles.  Valid values: `drivers`, `vehicles`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: "key:value". For example, "maintenanceId:250020".

        driver_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver tag IDs. Example: `tagIds=1234,5678`

        vehicle_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle tag IDs. Example: `tagIds=1234,5678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        assignment_type : typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType]
            Specifies which assignment type to filter by.  Valid values: `HOS`, `idCard`, `static`, `faceId`, `tachograph`, `safetyManual`, `RFID`, `trailer`, `external`, `qrCode`, `driverApp`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_vehicle_assignments.get_driver_vehicle_assignments(
            filter_by="drivers",
        )
        """
        _response = self._raw_client.get_driver_vehicle_assignments(
            filter_by=filter_by,
            start_time=start_time,
            end_time=end_time,
            driver_ids=driver_ids,
            vehicle_ids=vehicle_ids,
            driver_tag_ids=driver_tag_ids,
            vehicle_tag_ids=vehicle_tag_ids,
            after=after,
            assignment_type=assignment_type,
            request_options=request_options,
        )
        return _response.data

    def create_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody:
        """
        Assign vehicle drive-time to a driver via API. For a step-by-step instruction on how to leverage this endpoint, see [this guide](https://developers.samsara.com/docs/creating-driver-vehicle-assignments)

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. Defaults to max-time (meaning it's an ongoing assignment) if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger? Defaults to false if not provided

        metadata : typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody
            Created response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_vehicle_assignments.create_driver_vehicle_assignment(
            driver_id="494123",
            vehicle_id="281474978683353",
        )
        """
        _response = self._raw_client.create_driver_vehicle_assignment(
            driver_id=driver_id,
            vehicle_id=vehicle_id,
            assigned_at_time=assigned_at_time,
            end_time=end_time,
            is_passenger=is_passenger,
            metadata=metadata,
            start_time=start_time,
            request_options=request_options,
        )
        return _response.data

    def delete_driver_vehicle_assignments(
        self,
        *,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete driver assignments that were created using the `POST fleet/driver-vehicle-assignments` endpoint for the requested vehicle in the requested time range.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
             Assigned at time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Indicates if assigned driver is passenger

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_vehicle_assignments.delete_driver_vehicle_assignments(
            vehicle_id="281474978683353",
        )
        """
        _response = self._raw_client.delete_driver_vehicle_assignments(
            vehicle_id=vehicle_id,
            assigned_at_time=assigned_at_time,
            end_time=end_time,
            is_passenger=is_passenger,
            start_time=start_time,
            request_options=request_options,
        )
        return _response.data

    def update_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        start_time: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody:
        """
        Update driver assignments that were created using the `POST fleet/driver-vehicle-assignments`. Vehicle Id, Driver Id, and Start Time must match an existing assignment.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        start_time : str
            The start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. To make this an ongoing assignment (ie. an assignment with no end time), provide an endTime value of 'null'. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger?

        metadata : typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody
            Accepted response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.driver_vehicle_assignments.update_driver_vehicle_assignment(
            driver_id="494123",
            start_time="2019-06-13T19:08:25Z",
            vehicle_id="281474978683353",
        )
        """
        _response = self._raw_client.update_driver_vehicle_assignment(
            driver_id=driver_id,
            start_time=start_time,
            vehicle_id=vehicle_id,
            assigned_at_time=assigned_at_time,
            end_time=end_time,
            is_passenger=is_passenger,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data


class AsyncDriverVehicleAssignmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDriverVehicleAssignmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDriverVehicleAssignmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDriverVehicleAssignmentsClient
        """
        return self._raw_client

    async def get_driver_vehicle_assignments(
        self,
        *,
        filter_by: GetDriverVehicleAssignmentsRequestFilterBy,
        start_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        driver_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        vehicle_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        driver_tag_ids: typing.Optional[str] = None,
        vehicle_tag_ids: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        assignment_type: typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody:
        """
        Get all driver-vehicle assignments for the requested drivers or vehicles in the requested time range. To fetch driver-vehicle assignments out of the vehicle trips' time ranges, assignmentType needs to be specified. Note: this endpoint replaces past endpoints to fetch assignments by driver or by vehicle. Visit [this migration guide](https://developers.samsara.com/docs/migrating-from-driver-vehicle-assignment-or-vehicle-driver-assignment-endpoints) for more information.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        filter_by : GetDriverVehicleAssignmentsRequestFilterBy
            Option to filter by drivers or vehicles.  Valid values: `drivers`, `vehicles`

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        driver_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
             A filter on the data based on this comma-separated list of driver IDs and externalIds. Example: `driverIds=1234,5678,payroll:4841`

        vehicle_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            ID of the vehicle. This can either be the Samsara-specified ID, or an external ID. External IDs are customer specified key-value pairs created in the POST or PATCH requests of this resource. To specify an external ID as part of a path parameter, use the following format: "key:value". For example, "maintenanceId:250020".

        driver_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of driver tag IDs. Example: `tagIds=1234,5678`

        vehicle_tag_ids : typing.Optional[str]
             A filter on the data based on this comma-separated list of vehicle tag IDs. Example: `tagIds=1234,5678`

        after : typing.Optional[str]
             If specified, this should be the endCursor value from the previous page of results. When present, this request will return the next page of results that occur immediately after the previous page of results.

        assignment_type : typing.Optional[GetDriverVehicleAssignmentsRequestAssignmentType]
            Specifies which assignment type to filter by.  Valid values: `HOS`, `idCard`, `static`, `faceId`, `tachograph`, `safetyManual`, `RFID`, `trailer`, `external`, `qrCode`, `driverApp`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverVehicleAssignmentsV2GetDriverVehicleAssignmentsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_vehicle_assignments.get_driver_vehicle_assignments(
                filter_by="drivers",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_driver_vehicle_assignments(
            filter_by=filter_by,
            start_time=start_time,
            end_time=end_time,
            driver_ids=driver_ids,
            vehicle_ids=vehicle_ids,
            driver_tag_ids=driver_tag_ids,
            vehicle_tag_ids=vehicle_tag_ids,
            after=after,
            assignment_type=assignment_type,
            request_options=request_options,
        )
        return _response.data

    async def create_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody:
        """
        Assign vehicle drive-time to a driver via API. For a step-by-step instruction on how to leverage this endpoint, see [this guide](https://developers.samsara.com/docs/creating-driver-vehicle-assignments)

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. Defaults to max-time (meaning it's an ongoing assignment) if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger? Defaults to false if not provided

        metadata : typing.Optional[PostDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        start_time : typing.Optional[str]
            The start time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverVehicleAssignmentsV2CreateDriverVehicleAssignmentResponseBody
            Created response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_vehicle_assignments.create_driver_vehicle_assignment(
                driver_id="494123",
                vehicle_id="281474978683353",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_driver_vehicle_assignment(
            driver_id=driver_id,
            vehicle_id=vehicle_id,
            assigned_at_time=assigned_at_time,
            end_time=end_time,
            is_passenger=is_passenger,
            metadata=metadata,
            start_time=start_time,
            request_options=request_options,
        )
        return _response.data

    async def delete_driver_vehicle_assignments(
        self,
        *,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        start_time: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete driver assignments that were created using the `POST fleet/driver-vehicle-assignments` endpoint for the requested vehicle in the requested time range.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
             Assigned at time in RFC 3339 format. Defaults to now if not provided. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
             An end time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Indicates if assigned driver is passenger

        start_time : typing.Optional[str]
             A start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_vehicle_assignments.delete_driver_vehicle_assignments(
                vehicle_id="281474978683353",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_driver_vehicle_assignments(
            vehicle_id=vehicle_id,
            assigned_at_time=assigned_at_time,
            end_time=end_time,
            is_passenger=is_passenger,
            start_time=start_time,
            request_options=request_options,
        )
        return _response.data

    async def update_driver_vehicle_assignment(
        self,
        *,
        driver_id: str,
        start_time: str,
        vehicle_id: str,
        assigned_at_time: typing.Optional[str] = OMIT,
        end_time: typing.Optional[str] = OMIT,
        is_passenger: typing.Optional[bool] = OMIT,
        metadata: typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody:
        """
        Update driver assignments that were created using the `POST fleet/driver-vehicle-assignments`. Vehicle Id, Driver Id, and Start Time must match an existing assignment.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Assignments** under the Assignments category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_id : str
            ID of the driver. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the driver.

        start_time : str
            The start time in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        vehicle_id : str
            ID of the vehicle. This can be either a unique Samsara ID or an [external ID](https://developers.samsara.com/docs/external-ids) for the vehicle.

        assigned_at_time : typing.Optional[str]
            The time at which the assignment was made in RFC 3339 format. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        end_time : typing.Optional[str]
            The end time in RFC 3339 format. To make this an ongoing assignment (ie. an assignment with no end time), provide an endTime value of 'null'. Millisecond precision and timezones are supported. (Examples: 2019-06-13T19:08:25Z, 2019-06-13T19:08:25.455Z, OR 2015-09-15T14:00:12-04:00).

        is_passenger : typing.Optional[bool]
            Is this driver a passenger?

        metadata : typing.Optional[PatchDriverVehicleAssignmentsV2RequestBodyMetadataRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DriverVehicleAssignmentsV2UpdateDriverVehicleAssignmentResponseBody
            Accepted response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.driver_vehicle_assignments.update_driver_vehicle_assignment(
                driver_id="494123",
                start_time="2019-06-13T19:08:25Z",
                vehicle_id="281474978683353",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_driver_vehicle_assignment(
            driver_id=driver_id,
            start_time=start_time,
            vehicle_id=vehicle_id,
            assigned_at_time=assigned_at_time,
            end_time=end_time,
            is_passenger=is_passenger,
            metadata=metadata,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .parent_tag_id import ParentTagId
from .tag_id import TagId
from .tag_name import TagName


class TinyTag(UniversalBaseModel):
    id: typing.Optional[TagId] = None
    name: typing.Optional[TagName] = None
    parent_tag_id: typing_extensions.Annotated[
        typing.Optional[ParentTagId], FieldMetadata(alias="parentTagId"), pydantic.Field(alias="parentTagId")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .specific_vehicle_fault_code_object_request_body import SpecificVehicleFaultCodeObjectRequestBody


class VehicleFaultCodeDetailsObjectRequestBody(UniversalBaseModel):
    """
    Details specific to Vehicle Fault Code. At least one fault code or fault code group must be selected.
    """

    has_any_amber_warning_lamp_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasAnyAmberWarningLampCodes"),
        pydantic.Field(
            alias="hasAnyAmberWarningLampCodes",
            description="If true then alert on codes for less serious errors that do not warrant stopping. Defaults to false.",
        ),
    ] = None
    has_any_fault_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasAnyFaultCodes"),
        pydantic.Field(
            alias="hasAnyFaultCodes", description="If true this means that any code is alertable. Defaults to false."
        ),
    ] = None
    has_any_malfunction_indicator_lamp_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasAnyMalfunctionIndicatorLampCodes"),
        pydantic.Field(
            alias="hasAnyMalfunctionIndicatorLampCodes",
            description="If true then alert on emission-related codes. Defaults to false.",
        ),
    ] = None
    has_any_protection_lamp_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasAnyProtectionLampCodes"),
        pydantic.Field(
            alias="hasAnyProtectionLampCodes",
            description="If true then alert on codes for non-electric vehicle parts. Defaults to false.",
        ),
    ] = None
    has_any_red_stop_lamp_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasAnyRedStopLampCodes"),
        pydantic.Field(
            alias="hasAnyRedStopLampCodes",
            description="If true then alert when the vehicle warrants stopping. Defaults to false.",
        ),
    ] = None
    has_any_trailer_abs_lamp_codes: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="hasAnyTrailerAbsLampCodes"),
        pydantic.Field(
            alias="hasAnyTrailerAbsLampCodes",
            description="If true then alert when the ABS light is on. Defaults to false.",
        ),
    ] = None
    min_duration_milliseconds: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="minDurationMilliseconds"),
        pydantic.Field(
            alias="minDurationMilliseconds",
            description="The number of milliseconds the trigger needs to stay active before alerting.",
        ),
    ] = None
    specific_fault_codes: typing_extensions.Annotated[
        typing.Optional[typing.List[SpecificVehicleFaultCodeObjectRequestBody]],
        FieldMetadata(alias="specificFaultCodes"),
        pydantic.Field(alias="specificFaultCodes", description="The list of specific fault codes to be alerted on."),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .ridership_route_setup_passenger_object_response_body import RidershipRouteSetupPassengerObjectResponseBody


class RidershipRouteSetupObjectResponseBody(UniversalBaseModel):
    """
    A ridership route setup representing planned passenger assignments.
    """

    account_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="accountId"),
        pydantic.Field(alias="accountId", description="The Samsara UUID of the ridership account."),
    ]
    passengers: typing.List[RidershipRouteSetupPassengerObjectResponseBody] = pydantic.Field()
    """
    List of passenger assignments.
    """

    route_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="routeId"), pydantic.Field(alias="routeId", description="The route ID.")
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .training_assignment_near_due_date_trigger_assignment_group_object_response_body import (
    TrainingAssignmentNearDueDateTriggerAssignmentGroupObjectResponseBody,
)
from .training_assignment_near_due_date_trigger_details_object_response_body_condition_units import (
    TrainingAssignmentNearDueDateTriggerDetailsObjectResponseBodyConditionUnits,
)


class TrainingAssignmentNearDueDateTriggerDetailsObjectResponseBody(UniversalBaseModel):
    """
    Details specific to Training Assignment Near Due Date
    """

    assignment_groups: typing_extensions.Annotated[
        typing.Optional[typing.List[TrainingAssignmentNearDueDateTriggerAssignmentGroupObjectResponseBody]],
        FieldMetadata(alias="assignmentGroups"),
        pydantic.Field(alias="assignmentGroups", description="The assignment groups the trigger is configured for."),
    ] = None
    condition_units: typing_extensions.Annotated[
        TrainingAssignmentNearDueDateTriggerDetailsObjectResponseBodyConditionUnits,
        FieldMetadata(alias="conditionUnits"),
        pydantic.Field(
            alias="conditionUnits",
            description="Whether the trigger is configured in days or weeks.  Valid values: `DAYS`, `WEEKS`",
        ),
    ]
    condition_value: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="conditionValue"),
        pydantic.Field(
            alias="conditionValue", description="The number of days or weeks near the due date to trigger on."
        ),
    ]
    timezone: str = pydantic.Field()
    """
    The timezone that the alert will be set up in.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class DefectPhotoResponseResponseBody(UniversalBaseModel):
    """
    Defect's photo object
    """

    created_at_time: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(alias="createdAtTime", description="Time when defect's photo was created in RFC 3339 format."),
    ]
    url: str = pydantic.Field()
    """
    The URL to the defect's photo. Note: the link is available only for 24 hours after it's sent
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .forms_person_object_response_body_entry_type import FormsPersonObjectResponseBodyEntryType
from .forms_polymorphic_user_object_response_body import FormsPolymorphicUserObjectResponseBody


class FormsPersonObjectResponseBody(UniversalBaseModel):
    """
    Tracked or untracked (i.e. manually entered) person object.
    """

    entry_type: typing_extensions.Annotated[
        FormsPersonObjectResponseBodyEntryType,
        FieldMetadata(alias="entryType"),
        pydantic.Field(
            alias="entryType", description="The type of entry for the person.  Valid values: `tracked`, `untracked`"
        ),
    ]
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of an untracked (i.e. manually entered) person.
    """

    polymorphic_user_id: typing_extensions.Annotated[
        typing.Optional[FormsPolymorphicUserObjectResponseBody],
        FieldMetadata(alias="polymorphicUserId"),
        pydantic.Field(alias="polymorphicUserId"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .form_submission_response_object_response_body_status import FormSubmissionResponseObjectResponseBodyStatus
from .form_template_reference_object_response_body import FormTemplateReferenceObjectResponseBody
from .forms_asset_object_response_body import FormsAssetObjectResponseBody
from .forms_field_input_object_response_body import FormsFieldInputObjectResponseBody
from .forms_geofence_object_response_body import FormsGeofenceObjectResponseBody
from .forms_location_object_response_body import FormsLocationObjectResponseBody
from .forms_polymorphic_user_object_response_body import FormsPolymorphicUserObjectResponseBody
from .forms_product_submission_approval_details_object_response_body import (
    FormsProductSubmissionApprovalDetailsObjectResponseBody,
)
from .forms_score_object_response_body import FormsScoreObjectResponseBody


class FormSubmissionResponseObjectResponseBody(UniversalBaseModel):
    """
    Form Submission response object.
    """

    approval_details: typing_extensions.Annotated[
        typing.Optional[FormsProductSubmissionApprovalDetailsObjectResponseBody],
        FieldMetadata(alias="approvalDetails"),
        pydantic.Field(alias="approvalDetails"),
    ] = None
    asset: typing.Optional[FormsAssetObjectResponseBody] = None
    assigned_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="assignedAtTime"),
        pydantic.Field(
            alias="assignedAtTime",
            description="Assignment time of the form submission. Sometimes returned if the submission was assigned to a user or driver. UTC timestamp in RFC 3339 format.",
        ),
    ] = None
    assigned_to: typing_extensions.Annotated[
        typing.Optional[FormsPolymorphicUserObjectResponseBody],
        FieldMetadata(alias="assignedTo"),
        pydantic.Field(alias="assignedTo"),
    ] = None
    created_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="createdAtTime"),
        pydantic.Field(
            alias="createdAtTime", description="Creation time of the form submission. UTC timestamp in RFC 3339 format."
        ),
    ]
    due_at_time: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="dueAtTime"),
        pydantic.Field(
            alias="dueAtTime",
            description="Time of when the submission is due. Sometimes returned, if the submission has a due date. UTC timestamp in RFC 3339 format.",
        ),
    ] = None
    external_ids: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="externalIds"),
        pydantic.Field(alias="externalIds", description="A map of external ids"),
    ] = None
    fields: typing.List[FormsFieldInputObjectResponseBody] = pydantic.Field()
    """
    List of field inputs in a form submission.
    """

    form_template: typing_extensions.Annotated[
        FormTemplateReferenceObjectResponseBody,
        FieldMetadata(alias="formTemplate"),
        pydantic.Field(alias="formTemplate"),
    ]
    geofence: typing.Optional[FormsGeofenceObjectResponseBody] = None
    id: str = pydantic.Field()
    """
    ID of the form submission.
    """

    is_required: typing_extensions.Annotated[
        bool,
        FieldMetadata(alias="isRequired"),
        pydantic.Field(
            alias="isRequired",
            description="Indicates whether the worker is required to complete this form or not. Always returned.",
        ),
    ]
    location: typing.Optional[FormsLocationObjectResponseBody] = None
    route_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="routeId"),
        pydantic.Field(
            alias="routeId",
            description="ID of the route. Sometimes returned if the submission was assigned to a route stop.",
        ),
    ] = None
    route_stop_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="routeStopId"),
        pydantic.Field(
            alias="routeStopId",
            description="ID of the route stop. Sometimes returned if the submission was assigned to a route stop.",
        ),
    ] = None
    score: typing.Optional[FormsScoreObjectResponseBody] = None
    status: FormSubmissionResponseObjectResponseBodyStatus = pydantic.Field()
    """
    State for the Form Submission. Always returned.  Valid values: `notStarted`, `completed`, `archived`, `inProgress`, `needsReview`, `changesRequested`, `approved`
    """

    submitted_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="submittedAtTime"),
        pydantic.Field(
            alias="submittedAtTime",
            description="Submission time of the form submission. UTC timestamp in RFC 3339 format.",
        ),
    ]
    submitted_by: typing_extensions.Annotated[
        FormsPolymorphicUserObjectResponseBody, FieldMetadata(alias="submittedBy"), pydantic.Field(alias="submittedBy")
    ]
    title: typing.Optional[str] = pydantic.Field(default=None)
    """
    Title of the form submission. Sometimes returned if the submission has a title.
    """

    updated_at_time: typing_extensions.Annotated[
        dt.datetime,
        FieldMetadata(alias="updatedAtTime"),
        pydantic.Field(
            alias="updatedAtTime", description="Update time of the form submission. UTC timestamp in RFC 3339 format."
        ),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .fuel_energy_cost_response_response_body import FuelEnergyCostResponseResponseBody
from .fuel_energy_vehicle_object_response_body import FuelEnergyVehicleObjectResponseBody


class FuelEnergyVehicleReportObjectResponseBody(UniversalBaseModel):
    """
    A summary of this vehicle's fuel and energy data.
    """

    distance_traveled_meters: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="distanceTraveledMeters"),
        pydantic.Field(alias="distanceTraveledMeters", description="Meters traveled over the given time range."),
    ]
    efficiency_mpge: typing_extensions.Annotated[
        float,
        FieldMetadata(alias="efficiencyMpge"),
        pydantic.Field(
            alias="efficiencyMpge",
            description="Efficiency in MPG or MPGE over the given time range. For fuel vehicles this will be provided in MPG, for hybrid and electric vehicles this will be provided in MPGE. MPG/MPGE values are provided based on US gallons.",
        ),
    ]
    energy_used_kwh: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="energyUsedKwh"),
        pydantic.Field(
            alias="energyUsedKwh",
            description="Kilowatt-hours of energy used over the given time range. Only provided for hybrid and electric vehicles.",
        ),
    ] = None
    engine_idle_time_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="engineIdleTimeDurationMs"),
        pydantic.Field(
            alias="engineIdleTimeDurationMs",
            description="Milliseconds of engine idle time over the given time range. Only provided for fuel and hybrid vehicles.",
        ),
    ] = None
    engine_run_time_duration_ms: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="engineRunTimeDurationMs"),
        pydantic.Field(
            alias="engineRunTimeDurationMs",
            description="Milliseconds of engine run time over the given time range. Only provided for fuel and hybrid vehicles.",
        ),
    ] = None
    est_carbon_emissions_kg: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="estCarbonEmissionsKg"),
        pydantic.Field(
            alias="estCarbonEmissionsKg",
            description="Estimated kilograms of carbon emissions over the given time range. Only provided for fuel and hybrid vehicles.",
        ),
    ] = None
    est_fuel_energy_cost: typing_extensions.Annotated[
        FuelEnergyCostResponseResponseBody,
        FieldMetadata(alias="estFuelEnergyCost"),
        pydantic.Field(alias="estFuelEnergyCost"),
    ]
    fuel_consumed_ml: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="fuelConsumedMl"),
        pydantic.Field(
            alias="fuelConsumedMl",
            description="Milliliters of fuel consumed over the given time range. Only provided for fuel and hybrid vehicles.",
        ),
    ] = None
    vehicle: FuelEnergyVehicleObjectResponseBody

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .training_category_object_response_body import TrainingCategoryObjectResponseBody
from .training_course_label_object_response_body import TrainingCourseLabelObjectResponseBody
from .training_course_response_object_response_body_status import TrainingCourseResponseObjectResponseBodyStatus


class TrainingCourseResponseObjectResponseBody(UniversalBaseModel):
    """
    Training Course response object.
    """

    category: TrainingCategoryObjectResponseBody
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the course.
    """

    estimated_time_to_complete_minutes: typing_extensions.Annotated[
        int,
        FieldMetadata(alias="estimatedTimeToCompleteMinutes"),
        pydantic.Field(
            alias="estimatedTimeToCompleteMinutes", description="Estimated time it takes to complete the course."
        ),
    ]
    id: str = pydantic.Field()
    """
    ID of the training course.
    """

    labels: typing.Optional[typing.List[TrainingCourseLabelObjectResponseBody]] = pydantic.Field(default=None)
    """
    List of course labels.
    """

    revision_id: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="revisionId"),
        pydantic.Field(alias="revisionId", description="ID of the course's specific version."),
    ]
    status: TrainingCourseResponseObjectResponseBodyStatus = pydantic.Field()
    """
    Status of the training course. Always returned.
    Note: Only courses in a ‘published’ state are assignable to learners. Archiving a course deletes only ‘inProgress’ assignments associated with that course, and deleting a course deletes all assignments associated with that course.  Valid values: `published`, `deleted`, `archived`, `unknown`
    """

    title: str = pydantic.Field()
    """
    Title of the course.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
